@@ -23,30 +23,18 @@
 #include <Python.h>
 #include <tox/toxav.h>
 
+#include "av.h"
 #include "core.h"
 #include "util.h"
 
-/* ToxAV definition */
-typedef struct {
-  PyObject_HEAD
-  PyObject* core;
-  ToxAv* av;
-  int16_t* pcm;
-  vpx_image_t* in_image;
-  uint32_t o_w, o_h;
-  unsigned char* out_image;
-  ToxAvCodecSettings cs;
-} ToxAV;
-
 extern PyObject* ToxOpError;
 
-
 #define AUDIO_FRAME_SIZE (av_DefaultSettings.audio_sample_rate * av_DefaultSettings.audio_frame_duration / 1000)
 
-#define CALLBACK_DEF(name)                                   \
-  void ToxAV_callback_##name(void* self)                     \
-  {                                                          \
-    PyObject_CallMethod((PyObject*)self, #name, NULL);       \
+#define CALLBACK_DEF(name)                                          \
+  void ToxAV_callback_##name(void* agent, int32_t idx, void* self)  \
+  {                                                                 \
+    PyObject_CallMethod((PyObject*)self, #name, "i", idx);          \
   }
 
 CALLBACK_DEF(on_invite);
@@ -57,9 +45,9 @@ CALLBACK_DEF(on_end);
 CALLBACK_DEF(on_ringing);
 CALLBACK_DEF(on_starting);
 CALLBACK_DEF(on_ending);
-CALLBACK_DEF(on_error);
 CALLBACK_DEF(on_request_timeout);
 CALLBACK_DEF(on_peer_timeout);
+CALLBACK_DEF(on_media_change);
 
 static void i420_to_rgb(vpx_image_t *img, unsigned char *out)
 {
@@ -142,7 +130,6 @@ static void i420_to_rgb(vpx_image_t *img, unsigned char *out)
 
 static void rgb_to_i420(unsigned char* rgb, vpx_image_t *img)
 {
-  int image_size = img->d_w * img->d_h;
   int upos = 0;
   int vpos = 0;
   int x = 0, i = 0;
@@ -177,17 +164,73 @@ static void rgb_to_i420(unsigned char* rgb, vpx_image_t *img)
   }
 }
 
+void ToxAV_audio_recv_callback(ToxAv* av, int32_t call_idx, int16_t* data,
+    int size, void* self)
+{
+  PyGILState_STATE gstate = PyGILState_Ensure();
+
+  PyObject_CallMethod((PyObject*)self, "on_audio_data", "iis#", call_idx, size,
+      (char*)data, size << 1);
+
+  if (PyErr_Occurred()) {
+    PyErr_Print();
+  }
+
+  PyGILState_Release(gstate);
+}
+
+void ToxAV_video_recv_callback(ToxAv* av, int32_t call_idx, vpx_image_t* image,
+    void* user)
+{
+  ToxAV* self = (ToxAV*)user;
+
+  if (image == NULL) {
+    return;
+  }
+
+  if (self->out_image && (self->o_w != image->d_w || self->o_h != image->d_h)) {
+    free(self->out_image);
+    self->out_image = NULL;
+  }
+
+  const int buf_size = image->d_w * image->d_h * 3;
+
+  if (self->out_image == NULL) {
+    self->o_w = image->d_w;
+    self->o_h = image->d_h;
+    self->out_image = malloc(buf_size);
+  }
+
+  i420_to_rgb(image, self->out_image);
+
+  vpx_img_free(image);
+
+  PyGILState_STATE gstate = PyGILState_Ensure();
+
+  PyObject_CallMethod((PyObject*)self, "on_video_data", "iiis#", call_idx,
+      self->o_w, self->o_h, self->out_image, buf_size);
+
+  if (PyErr_Occurred()) {
+    PyErr_Print();
+  }
+
+  PyGILState_Release(gstate);
+}
+
+
 static int init_helper(ToxAV* self, PyObject* args)
 {
+  PyEval_InitThreads();
+
   if (self->av) {
     toxav_kill(self->av);
     self->av = NULL;
   }
 
   PyObject* core = NULL;
-  int width = 0, height = 0;
+  int max_calls;
 
-  if (!PyArg_ParseTuple(args, "Oii", &core, &width, &height)) {
+  if (!PyArg_ParseTuple(args, "Oi", &core, &max_calls)) {
     PyErr_SetString(PyExc_TypeError, "must associate with a core instance");
     return -1;
   }
@@ -195,36 +238,32 @@ static int init_helper(ToxAV* self, PyObject* args)
   self->core = core;
   Py_INCREF(self->core);
 
-  self->cs = av_DefaultSettings;
-  self->cs.video_height = height;
-  self->cs.video_width = width;
+  self->av = toxav_new(((ToxCore*)self->core)->tox, max_calls);
 
-  self->av = toxav_new(((ToxCore*)self->core)->tox, &self->cs);
+  self->cs = av_DefaultSettings;
+  self->cs.max_video_width = self->cs.max_video_height = 0;
 
 #define REG_CALLBACK(id, name) \
-  toxav_register_callstate_callback(ToxAV_callback_##name, id, self)
+  toxav_register_callstate_callback(self->av, ToxAV_callback_##name, id, self)
 
   REG_CALLBACK(av_OnInvite, on_invite);
   REG_CALLBACK(av_OnStart, on_start);
   REG_CALLBACK(av_OnCancel, on_cancel);
   REG_CALLBACK(av_OnReject, on_reject);
   REG_CALLBACK(av_OnEnd, on_end);
+
   REG_CALLBACK(av_OnRinging, on_ringing);
   REG_CALLBACK(av_OnStarting, on_starting);
   REG_CALLBACK(av_OnEnding, on_ending);
-  REG_CALLBACK(av_OnError, on_error);
+
   REG_CALLBACK(av_OnRequestTimeout, on_request_timeout);
   REG_CALLBACK(av_OnPeerTimeout, on_peer_timeout);
+  REG_CALLBACK(av_OnMediaChange, on_media_change);
 
 #undef REG_CALLBACK
 
-
-  if (self->in_image == NULL) {
-    self->in_image = vpx_img_alloc(NULL, VPX_IMG_FMT_I420, width, height, 1);
-  }
-  if (self->pcm == NULL) {
-    self->pcm = (int16_t*)malloc(AUDIO_FRAME_SIZE * sizeof(int16_t));
-  }
+  toxav_register_audio_recv_callback(self->av, ToxAV_audio_recv_callback, self);
+  toxav_register_video_recv_callback(self->av, ToxAV_video_recv_callback, self);
 
   if (self->av == NULL) {
     PyErr_SetString(ToxOpError, "failed to allocate toxav");
@@ -241,7 +280,7 @@ ToxAV_new(PyTypeObject *type, PyObject* args, PyObject* kwds)
   self->av = NULL;
   self->in_image = NULL;
   self->out_image = NULL;
-  self->pcm = NULL;
+  self->i_w = self->i_h = self->o_w = self->o_h = 0;
 
   if (init_helper(self, args) == -1) {
     return NULL;
@@ -267,7 +306,10 @@ ToxAV_dealloc(ToxAV* self)
     Py_DECREF(self->core);
     toxav_kill(self->av);
     self->av = NULL;
-    vpx_img_free(self->in_image);
+
+    if (self->in_image) {
+      vpx_img_free(self->in_image);
+    }
     if (self->out_image) {
       free(self->out_image);
     }
@@ -310,6 +352,12 @@ ToxAV_set_Error(int ret)
   case ErrorTerminatingVideoRtp:
     msg = "Returned in toxav_kill_transmission()";
     break;
+  case ErrorPacketTooLarge:
+    msg = "Buffer exceeds size while encoding";
+    break;
+  case ErrorInvalidCodecState:
+    msg = "Codec state not initialized";
+    break;
   default:
     msg = "Unknown error";
   }
@@ -321,26 +369,32 @@ static PyObject*
 ToxAV_call(ToxAV* self, PyObject* args)
 {
   int peer_id = 0;
-  int call_type = 0;
   int seconds = 0;
 
-  if (!PyArg_ParseTuple(args, "iii", &peer_id, &call_type, &seconds)) {
+  if (!PyArg_ParseTuple(args, "iii", &peer_id, &self->cs.call_type, &seconds)) {
     return NULL;
   }
 
-  int ret = toxav_call(self->av, peer_id, call_type, seconds);
+  int32_t call_idx = 0;
+  int ret = toxav_call(self->av, &call_idx, peer_id, &self->cs, seconds);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
   }
 
-  Py_RETURN_NONE;
+  return PyLong_FromLong(call_idx);
 }
 
 static PyObject*
 ToxAV_hangup(ToxAV* self, PyObject* args)
 {
-  int ret = toxav_hangup(self->av);
+  int idx = 0;
+
+  if (!PyArg_ParseTuple(args, "i", &idx)) {
+    return NULL;
+  }
+
+  int ret = toxav_hangup(self->av, idx);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -352,13 +406,13 @@ ToxAV_hangup(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_answer(ToxAV* self, PyObject* args)
 {
-  int call_type = 0;
+  int32_t idx = 0;
 
-  if (!PyArg_ParseTuple(args, "i", &call_type)) {
+  if (!PyArg_ParseTuple(args, "ii", &idx, &self->cs.call_type)) {
     return NULL;
   }
 
-  int ret = toxav_answer(self->av, call_type);
+  int ret = toxav_answer(self->av, idx, &self->cs);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -370,13 +424,14 @@ ToxAV_answer(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_reject(ToxAV* self, PyObject* args)
 {
+  int idx = 0;
   const char* res = NULL;
 
-  if (!PyArg_ParseTuple(args, "s", &res)) {
+  if (!PyArg_ParseTuple(args, "is", &idx, &res)) {
     return NULL;
   }
 
-  int ret = toxav_reject(self->av, res);
+  int ret = toxav_reject(self->av, idx, res);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -388,14 +443,56 @@ ToxAV_reject(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_cancel(ToxAV* self, PyObject* args)
 {
-  int peer_id = 0;
+  int idx = 0, peer_id = 0;
   const char* res = NULL;
 
-  if (!PyArg_ParseTuple(args, "is", &peer_id, &res)) {
+  if (!PyArg_ParseTuple(args, "iis", &idx, &peer_id, &res)) {
     return NULL;
   }
 
-  int ret = toxav_cancel(self->av, peer_id, res);
+  int ret = toxav_cancel(self->av, idx, peer_id, res);
+  if (ret < 0) {
+    ToxAV_set_Error(ret);
+    return NULL;
+  }
+
+  Py_RETURN_NONE;
+}
+
+static PyObject*
+ToxAV_change_settings(ToxAV* self, PyObject* args)
+{
+  int idx = 0;
+  PyObject* settings = NULL;
+
+  if (!PyArg_ParseTuple(args, "iO", &idx, &settings)) {
+    return NULL;
+  }
+
+  if (!PyDict_Check(settings)) {
+    PyErr_SetString(PyExc_TypeError, "settings should be a dictionary");
+    return NULL;
+  }
+
+#define CHECK_AND_UPDATE(d, name)                                    \
+  PyObject* key_##name = PYSTRING_FromString(#name);                 \
+  if (PyDict_Contains(d, key_##name)) {                              \
+    self->cs.name = PyLong_AsLong(PyDict_GetItemString(d, #name));   \
+  }                                                                  \
+  Py_DECREF(key_##name);
+
+  CHECK_AND_UPDATE(settings, call_type);
+  CHECK_AND_UPDATE(settings, video_bitrate);
+  CHECK_AND_UPDATE(settings, max_video_width);
+  CHECK_AND_UPDATE(settings, max_video_height);
+  CHECK_AND_UPDATE(settings, audio_bitrate);
+  CHECK_AND_UPDATE(settings, audio_frame_duration);
+  CHECK_AND_UPDATE(settings, audio_sample_rate);
+  CHECK_AND_UPDATE(settings, audio_channels);
+
+#undef CHECK_AND_UPDATE
+
+  int ret = toxav_change_settings(self->av, idx, &self->cs);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -407,7 +504,13 @@ ToxAV_cancel(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_stop_call(ToxAV* self, PyObject* args)
 {
-  int ret = toxav_stop_call(self->av);
+  int idx = 0;
+
+  if (!PyArg_ParseTuple(args, "i", &idx)) {
+    return NULL;
+  }
+
+  int ret = toxav_stop_call(self->av, idx);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -419,13 +522,16 @@ ToxAV_stop_call(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_prepare_transmission(ToxAV* self, PyObject* args)
 {
-  int support_video = 0;
+  int idx = 0, support_video = 0;
+  uint32_t jbuf_size = 0, VAD_threshold = 0;
 
-  if (!PyArg_ParseTuple(args, "i", &support_video)) {
+  if (!PyArg_ParseTuple(args, "iIIi", &idx, &jbuf_size, &VAD_threshold,
+        &support_video)) {
     return NULL;
   }
 
-  int ret = toxav_prepare_transmission(self->av, support_video);
+  int ret = toxav_prepare_transmission(self->av, idx, jbuf_size, VAD_threshold,
+      support_video);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -437,7 +543,13 @@ ToxAV_prepare_transmission(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_kill_transmission(ToxAV* self, PyObject* args)
 {
-  int ret = toxav_kill_transmission(self->av);
+  int idx = 0;
+
+  if (!PyArg_ParseTuple(args, "i", &idx)) {
+    return NULL;
+  }
+
+  int ret = toxav_kill_transmission(self->av, idx);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -446,93 +558,34 @@ ToxAV_kill_transmission(ToxAV* self, PyObject* args)
   Py_RETURN_NONE;
 }
 
-static PyObject*
-ToxAV_recv_video(ToxAV* self, PyObject* args)
-{
-  vpx_image_t* image = NULL;
-
-  int ret = toxav_recv_video(self->av, &image);
-  if (ret < 0) {
-    ToxAV_set_Error(ret);
-    return NULL;
-  }
-
-  if (image == NULL) {
-    Py_RETURN_NONE;
-  }
-
-  if (self->out_image && (self->o_w != image->d_w || self->o_h != image->d_h)) {
-    free(self->out_image);
-    self->out_image = NULL;
-  }
-
-  if (self->out_image == NULL) {
-    self->o_w = image->d_w;
-    self->o_h = image->d_h;
-    self->out_image = malloc(self->o_w * self->o_h * 3);
-  }
-
-  i420_to_rgb(image, self->out_image);
-
-  PyObject* d = PyDict_New();
-  PyObject* d_w = PyLong_FromLong(image->d_w);
-  PyDict_SetItemString(d, "d_w", d_w);
-  Py_DECREF(d_w);
-
-  PyObject* d_h = PyLong_FromLong(image->d_h);
-  PyDict_SetItemString(d, "d_h", d_h);
-  Py_DECREF(d_h);
-
-  PyObject* data =
-      PYBYTES_FromStringAndSize(self->out_image, image->d_w * image->d_h * 3);
-  PyDict_SetItemString(d, "data", data);
-  Py_DECREF(data);
-
-  vpx_img_free(image);
-
-  return d;
-}
-
-static PyObject*
-ToxAV_recv_audio(ToxAV* self, PyObject* args)
-{
-  int ret = toxav_recv_audio(self->av, AUDIO_FRAME_SIZE, self->pcm);
-  if (ret < 0) {
-    ToxAV_set_Error(ret);
-    return NULL;
-  }
-
-  if (ret == 0) {
-    Py_RETURN_NONE;
-  }
-
-  PyObject* d = PyDict_New();
-
-  PyObject* size = PyLong_FromLong(ret);
-  PyDict_SetItemString(d, "size", size);
-  Py_DECREF(size);
-
-  PyObject* data =
-    PYBYTES_FromStringAndSize((char*)self->pcm, AUDIO_FRAME_SIZE << 1);
-  PyDict_SetItemString(d, "data", data);
-  Py_DECREF(data);
-
-  return d;
-}
-
 static PyObject*
 ToxAV_send_video(ToxAV* self, PyObject* args)
 {
-  int len = 0;
+  int idx = 0, len = 0, w = 0, h = 0;
   char* data = NULL;
 
-  if (!PyArg_ParseTuple(args, "s#", &data, &len)) {
+  if (!PyArg_ParseTuple(args, "iiis#", &idx, &w, &h, &data, &len)) {
     return NULL;
   }
 
-  rgb_to_i420(data, self->in_image);
+  if (self->in_image && (self->i_w != w || self->i_h != h)) {
+    vpx_img_free(self->in_image);
+    self->in_image = NULL;
+  }
 
-  int ret = toxav_send_video(self->av, self->in_image);
+  if (self->in_image == NULL) {
+    self->i_w = w;
+    self->i_h = h;
+    self->in_image = vpx_img_alloc(NULL, VPX_IMG_FMT_I420, w, h, 1);
+  }
+
+  rgb_to_i420((unsigned char*)data, self->in_image);
+
+  uint8_t encoded_payload[RTP_PAYLOAD_SIZE];
+  int32_t payload_size = toxav_prepare_video_frame(self->av, idx,
+      encoded_payload, RTP_PAYLOAD_SIZE, self->in_image);
+
+  int ret = toxav_send_video(self->av, idx, encoded_payload, payload_size);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -544,14 +597,18 @@ ToxAV_send_video(ToxAV* self, PyObject* args)
 static PyObject*
 ToxAV_send_audio(ToxAV* self, PyObject* args)
 {
-  int frame_size = 0, len = 0;
+  int idx = 0, frame_size = 0, len = 0;
   char* data = NULL;
 
-  if (!PyArg_ParseTuple(args, "is#",  &frame_size, &data, &len)) {
+  if (!PyArg_ParseTuple(args, "iis#", &idx, &frame_size, &data, &len)) {
     return NULL;
   }
 
-  int ret = toxav_send_audio(self->av, (int16_t*)data, frame_size);
+  uint8_t encoded_payload[RTP_PAYLOAD_SIZE];
+  int32_t payload_size = toxav_prepare_audio_frame(self->av, idx,
+      encoded_payload, RTP_PAYLOAD_SIZE, (int16_t*)data, frame_size);
+
+  int ret = toxav_send_audio(self->av, idx, encoded_payload, payload_size);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -561,33 +618,54 @@ ToxAV_send_audio(ToxAV* self, PyObject* args)
 }
 
 static PyObject*
-ToxAV_get_peer_transmission_type(ToxAV* self, PyObject* args)
+ToxAV_get_peer_csettings(ToxAV* self, PyObject* args)
 {
-  int peer = 0;
+  int idx = 0, peer = 0;
 
-  if (!PyArg_ParseTuple(args, "i", &peer)) {
+  if (!PyArg_ParseTuple(args, "ii", &idx, &peer)) {
     return NULL;
   }
 
-  int ret = toxav_get_peer_transmission_type(self->av, peer);
+  ToxAvCSettings cs;
+  int ret = toxav_get_peer_csettings(self->av, idx, peer, &cs);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
   }
 
-  return PyLong_FromLong(ret);
+#define SET_VALUE(d, attr)                                         \
+  PyObject* attr = PyLong_FromLong(cs.attr);                       \
+  PyDict_SetItemString(d, #attr, attr);                            \
+  Py_DECREF(attr);
+
+  PyObject* d = PyDict_New();
+
+  SET_VALUE(d, call_type);
+
+  SET_VALUE(d, video_bitrate);
+  SET_VALUE(d, max_video_width);
+  SET_VALUE(d, max_video_height);
+
+  SET_VALUE(d, audio_bitrate);
+  SET_VALUE(d, audio_frame_duration);
+  SET_VALUE(d, audio_sample_rate);
+  SET_VALUE(d, audio_channels);
+
+#undef SET_VALUE
+
+  return d;
 }
 
 static PyObject*
 ToxAV_get_peer_id(ToxAV* self, PyObject* args)
 {
-  int peer = 0;
+  int idx = 0, peer = 0;
 
-  if (!PyArg_ParseTuple(args, "i", &peer)) {
+  if (!PyArg_ParseTuple(args, "ii", &idx, &peer)) {
     return NULL;
   }
 
-  int ret = toxav_get_peer_id(self->av, peer);
+  int ret = toxav_get_peer_id(self->av, idx, peer);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -597,15 +675,33 @@ ToxAV_get_peer_id(ToxAV* self, PyObject* args)
 }
 
 static PyObject*
-ToxAV_capability_supported(ToxAV* self, PyObject* args)
+ToxAV_get_call_state(ToxAV* self, PyObject* args)
 {
-  int cap = 0;
+  int idx = 0;
 
-  if (!PyArg_ParseTuple(args, "i", &cap)) {
+  if (!PyArg_ParseTuple(args, "i", &idx)) {
     return NULL;
   }
 
-  int ret = toxav_capability_supported(self->av, cap);
+  int ret = toxav_get_call_state(self->av, idx);
+  if (ret < 0) {
+    ToxAV_set_Error(ret);
+    return NULL;
+  }
+
+  return PyBool_FromLong(ret);
+}
+
+static PyObject*
+ToxAV_capability_supported(ToxAV* self, PyObject* args)
+{
+  int idx = 0, cap = 0;
+
+  if (!PyArg_ParseTuple(args, "ii", &idx, &cap)) {
+    return NULL;
+  }
+
+  int ret = toxav_capability_supported(self->av, idx, cap);
   if (ret < 0) {
     ToxAV_set_Error(ret);
     return NULL;
@@ -627,9 +723,11 @@ ToxAV_callback_stub(ToxAV* self, PyObject* args)
   Py_RETURN_NONE;
 }
 
-#define METHOD_DEF(name)                                     \
-  {                                                          \
-    #name, (PyCFunction)ToxAV_callback_stub, METH_NOARGS, "" \
+#define METHOD_DEF(name)                                   \
+  {                                                        \
+    #name, (PyCFunction)ToxAV_callback_stub, METH_VARARGS, \
+    #name "\n"                                             \
+    #name " handler. Default implementation does nothing." \
   }
 
 PyMethodDef ToxAV_methods[] = {
@@ -644,58 +742,129 @@ PyMethodDef ToxAV_methods[] = {
   METHOD_DEF(on_error),
   METHOD_DEF(on_request_timeout),
   METHOD_DEF(on_peer_timeout),
+  METHOD_DEF(on_media_change),
   {
-    "call", (PyCFunction)ToxAV_call, METH_VARARGS, ""
+    "call", (PyCFunction)ToxAV_call, METH_VARARGS,
+    "call(friend_id, call_type, seconds)\n"
+    "Call a friend with *friend_id*, with *seconds* ringing timeout.\n\n"
+    "*call_type* can be one of following value:\n\n"
+    "+------------------+-----------------------+\n"
+    "| control_type     | description           |\n"
+    "+==================+=======================+\n"
+    "| ToxAV.TypeAudio  | audio only call       |\n"
+    "+------------------+-----------------------+\n"
+    "| ToxAV.TypeVideo  | audio and video call  |\n"
+    "+------------------+-----------------------+\n"
   },
   {
-    "hangup", (PyCFunction)ToxAV_hangup, METH_NOARGS, ""
+    "hangup", (PyCFunction)ToxAV_hangup, METH_NOARGS,
+    "hangup()\n"
+    "Hangup active call."
   },
   {
-    "answer", (PyCFunction)ToxAV_answer, METH_VARARGS, ""
+    "answer", (PyCFunction)ToxAV_answer, METH_VARARGS,
+    "answer(idx, call_type)\n"
+    "Answer incomming call.\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.call`"
   },
   {
-    "reject", (PyCFunction)ToxAV_reject, METH_VARARGS, ""
+    "reject", (PyCFunction)ToxAV_reject, METH_VARARGS,
+    "reject(idx)\n"
+    "Reject incomming call."
   },
   {
-    "cancel", (PyCFunction)ToxAV_cancel, METH_VARARGS, ""
+    "cancel", (PyCFunction)ToxAV_cancel, METH_VARARGS,
+    "cancel(idx)\n"
+    "Cancel outgoing request."
   },
   {
-    "stop_call", (PyCFunction)ToxAV_stop_call, METH_NOARGS, ""
+    "change_settings", (PyCFunction)ToxAV_change_settings, METH_VARARGS,
+    "change_settings(idx, csettings)\n"
+    "Notify peer that we are changing call settings. *settings* is a"
+    "dictionary of new settings like the one returned by "
+    "get_peer_csettings\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.get_peer_csettings`"
+  },
+  {
+    "stop_call", (PyCFunction)ToxAV_stop_call, METH_NOARGS,
+    "stop_call(idx)\n"
+    "Terminate transmission. Note that transmission will be terminated "
+    "without informing remote peer."
   },
   {
     "prepare_transmission", (PyCFunction)ToxAV_prepare_transmission,
-    METH_VARARGS, ""
+    METH_VARARGS,
+    "prepare_transmission(idx, jbuf_size, VAD_threshold, support_video)\n"
+    "Must be call before any RTP transmission occurs. *support_video* is "
+    "either True or False."
   },
   {
     "kill_transmission", (PyCFunction)ToxAV_kill_transmission,
-    METH_NOARGS, ""
+    METH_NOARGS,
+    "kill_transmission(idx)\n"
+    "Call this at the end of the transmission."
   },
   {
-    "recv_video", (PyCFunction)ToxAV_recv_video, METH_VARARGS, ""
+    "on_video_data", (PyCFunction)ToxAV_callback_stub, METH_VARARGS,
+    "on_video_data(idx, width, height, data)\n"
+    "Receive decoded video packet. Default implementation does nothing."
   },
   {
-    "recv_audio", (PyCFunction)ToxAV_recv_audio, METH_VARARGS, ""
+    "on_audio_data", (PyCFunction)ToxAV_callback_stub, METH_VARARGS,
+    "on_audio_data(idx, size, data)\n"
+    "Receive decoded audio packet. Default implementation does nothing."
   },
   {
-    "send_video", (PyCFunction)ToxAV_send_video, METH_VARARGS, ""
+    "send_video", (PyCFunction)ToxAV_send_video, METH_VARARGS,
+    "send_video(idx, data)\n"
+    "Encode and send video packet. *data* should be a str or buffer"
+    "containing a image in RGB888 format."
   },
   {
-    "send_audio", (PyCFunction)ToxAV_send_audio, METH_VARARGS, ""
+    "send_audio", (PyCFunction)ToxAV_send_audio, METH_VARARGS,
+    "send_audio(idx, frame_size, data)\n"
+    "Encode and send video packet. *data* should be a str or buffer"
+    "containing singal channel 16 bit signed PCM audio data."
   },
   {
-    "get_peer_transmission_type",
-    (PyCFunction)ToxAV_get_peer_transmission_type, METH_VARARGS, ""
+    "get_peer_csettings",
+    (PyCFunction)ToxAV_get_peer_csettings, METH_VARARGS,
+    "get_peer_csettings(idx, peer_num)\n"
+    "Get peer transmission type. It can either be audio or video."
+    "*peer_num* is always 0 for now."
   },
   {
-    "get_peer_id", (PyCFunction)ToxAV_get_peer_id, METH_VARARGS, ""
+    "get_peer_id", (PyCFunction)ToxAV_get_peer_id, METH_VARARGS,
+    "get_peer_id(peer_num)\n"
+    "Get *friend_number* of peer participating in conversation. *peer_num* "
+    "is always 0 for now."
+
   },
   {
     "capability_supported", (PyCFunction)ToxAV_capability_supported,
-    METH_VARARGS, ""
+    METH_VARARGS,
+    "capability_supported(capability)\n"
+    "Query if certain capability is supported.\n\n"
+    "*capability* can be one of following value:\n\n"
+    "+----------------------+-----------------+\n"
+    "| control_type         | description     |\n"
+    "+======================+=================+\n"
+    "| ToxAV.AudioEncoding  | audio encoding  |\n"
+    "+----------------------+-----------------+\n"
+    "| ToxAV.AudioDecoding  | audio decoding  |\n"
+    "+----------------------+-----------------+\n"
+    "| ToxAV.VideoEncoding  | video encoding  |\n"
+    "+----------------------+-----------------+\n"
+    "| ToxAV.VideoEncoding  | video decoding  |\n"
+    "+----------------------+-----------------+\n"
   },
   {
     "get_tox", (PyCFunction)ToxAV_get_tox,
-    METH_VARARGS, ""
+    METH_VARARGS,
+    "get_tox()\n"
+    "Get the Tox object associated with this ToxAV instance."
   },
 };
 
@@ -753,15 +922,31 @@ void ToxAV_install_dict()
   PyDict_SetItemString(dict, #name, obj_##name);             \
   Py_DECREF(obj_##name);
 
+#define SET2(name)                                           \
+  PyObject* obj_##name = PyLong_FromLong(av_##name);         \
+  PyDict_SetItemString(dict, #name, obj_##name);             \
+  Py_DECREF(obj_##name);
+
   PyObject* dict = PyDict_New();
   SET(TypeAudio);
   SET(TypeVideo);
-  SET(None);
   SET(AudioEncoding);
   SET(AudioDecoding);
   SET(VideoEncoding);
   SET(VideoDecoding);
 
+  SET2(CallNonExistant);
+  SET2(CallInviting);
+  SET2(CallStarting);
+  SET2(CallActive);
+  SET2(CallHold);
+  SET2(CallHanged_up);
+
+  SET2(jbufdc);
+  SET2(VADd);
+
+#undef SET2
+
 #undef SET
 
   ToxAVType.tp_dict = dict;
