@@ -23,46 +23,48 @@
 #include "core.h"
 #include "util.h"
 
+#include <arpa/inet.h>
+
 extern PyObject* ToxOpError;
 
-static void callback_friend_request(Tox* tox, uint8_t* public_key,
-    uint8_t* data, uint16_t length, void* self)
+static void callback_friend_request(Tox* tox, const uint8_t* public_key,
+    const uint8_t* data, uint16_t length, void* self)
 {
-  uint8_t buf[TOX_CLIENT_ID_SIZE * 2 + 1];
-  memset(buf, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
+  uint8_t buf[TOX_CLIENT_ID_SIZE << 1 + 1];
+  memset(buf, 0, TOX_CLIENT_ID_SIZE << 1 + 1);
 
   bytes_to_hex_string(public_key, TOX_CLIENT_ID_SIZE, buf);
 
   PyObject_CallMethod((PyObject*)self, "on_friend_request", "ss#", buf, data,
-      length);
+      length - (data[length - 1] == 0));
 }
 
 static void callback_friend_message(Tox *tox, int32_t friendnumber,
-    uint8_t* message, uint16_t length, void* self)
+    const uint8_t* message, uint16_t length, void* self)
 {
   PyObject_CallMethod((PyObject*)self, "on_friend_message", "is#", friendnumber,
-      message, length);
+      message, length - (message[length - 1] == 0));
 }
 
-static void callback_action(Tox *tox, int32_t friendnumber, uint8_t* action,
-    uint16_t length, void* self)
+static void callback_action(Tox *tox, int32_t friendnumber,
+    const uint8_t* action, uint16_t length, void* self)
 {
   PyObject_CallMethod((PyObject*)self, "on_friend_action", "is#", friendnumber,
-      action, length);
+      action, length - (action[length - 1] == 0));
 }
 
 static void callback_name_change(Tox *tox, int32_t friendnumber,
-    uint8_t* newname, uint16_t length, void* self)
+    const uint8_t* newname, uint16_t length, void* self)
 {
   PyObject_CallMethod((PyObject*)self, "on_name_change", "is#", friendnumber,
-      newname, length);
+      newname, length - (newname[length - 1] == 0));
 }
 
 static void callback_status_message(Tox *tox, int32_t friendnumber,
-    uint8_t *newstatus, uint16_t length, void* self)
+    const uint8_t *newstatus, uint16_t length, void* self)
 {
   PyObject_CallMethod((PyObject*)self, "on_status_message", "is#", friendnumber,
-      newstatus, length);
+      newstatus, length - (newstatus[length - 1] == 0));
 }
 
 static void callback_user_status(Tox *tox, int32_t friendnumber,
@@ -94,10 +96,10 @@ static void callback_connection_status(Tox *tox, int32_t friendnumber,
 }
 
 static void callback_group_invite(Tox *tox, int32_t friendnumber,
-    uint8_t* group_public_key, void *self)
+    const uint8_t* group_public_key, void *self)
 {
-  uint8_t gpk[TOX_CLIENT_ID_SIZE * 2 + 1];
-  memset(gpk, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
+  uint8_t gpk[TOX_CLIENT_ID_SIZE << 1 + 1];
+  memset(gpk, 0, TOX_CLIENT_ID_SIZE << 1 + 1);
 
   bytes_to_hex_string(group_public_key, TOX_CLIENT_ID_SIZE, gpk);
   PyObject_CallMethod((PyObject*)self, "on_group_invite", "is", friendnumber,
@@ -105,17 +107,17 @@ static void callback_group_invite(Tox *tox, int32_t friendnumber,
 }
 
 static void callback_group_message(Tox *tox, int groupid,
-    int friendgroupid, uint8_t* message, uint16_t length, void *self)
+    int friendgroupid, const uint8_t* message, uint16_t length, void *self)
 {
   PyObject_CallMethod((PyObject*)self, "on_group_message", "iis#", groupid,
-      friendgroupid, message, length);
+      friendgroupid, message, length - (message[length - 1] == 0));
 }
 
 static void callback_group_action(Tox *tox, int groupid,
-    int friendgroupid, uint8_t* action, uint16_t length, void *self)
+    int friendgroupid, const uint8_t* action, uint16_t length, void *self)
 {
   PyObject_CallMethod((PyObject*)self, "on_group_action", "iis#", groupid,
-      friendgroupid, action, length);
+      friendgroupid, action, length - (action[length - 1] == 0));
 }
 
 static void callback_group_namelist_change(Tox *tox, int groupid,
@@ -126,17 +128,17 @@ static void callback_group_namelist_change(Tox *tox, int groupid,
 }
 
 static void callback_file_send_request(Tox *m, int32_t friendnumber,
-    uint8_t filenumber, uint64_t filesize, uint8_t* filename,
+    uint8_t filenumber, uint64_t filesize, const uint8_t* filename,
     uint16_t filename_length, void* self)
 {
   PyObject_CallMethod((PyObject*)self, "on_file_send_request", "iiKs#",
       friendnumber, filenumber, filesize, filename,
-      filename_length);
+      filename_length - (filename[filename_length - 1] == 0));
 }
 
 static void callback_file_control(Tox *m, int32_t friendnumber,
     uint8_t receive_send, uint8_t filenumber, uint8_t control_type,
-    uint8_t* data, uint16_t length, void* self)
+    const uint8_t* data, uint16_t length, void* self)
 {
 #if PY_MAJOR_VERSION < 3
   PyObject_CallMethod((PyObject*)self, "on_file_control", "iiiis#",
@@ -148,7 +150,7 @@ static void callback_file_control(Tox *m, int32_t friendnumber,
 }
 
 static void callback_file_data(Tox *m, int32_t friendnumber, uint8_t filenumber,
-    uint8_t* data, uint16_t length, void* self)
+    const uint8_t* data, uint16_t length, void* self)
 {
 #if PY_MAJOR_VERSION < 3
   PyObject_CallMethod((PyObject*)self, "on_file_data", "iis#",
@@ -254,8 +256,8 @@ ToxCore_getaddress(ToxCore* self, PyObject* args)
   CHECK_TOX(self);
 
   uint8_t address[TOX_FRIEND_ADDRESS_SIZE];
-  uint8_t address_hex[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
-  memset(address_hex, 0, TOX_FRIEND_ADDRESS_SIZE * 2 + 1);
+  uint8_t address_hex[TOX_FRIEND_ADDRESS_SIZE << 1 + 1];
+  memset(address_hex, 0, TOX_FRIEND_ADDRESS_SIZE << 1 + 1);
 
   tox_get_address(self->tox, address);
   bytes_to_hex_string(address, TOX_FRIEND_ADDRESS_SIZE, address_hex);
@@ -378,8 +380,8 @@ ToxCore_get_client_id(ToxCore* self, PyObject* args)
   uint8_t pk[TOX_CLIENT_ID_SIZE + 1];
   pk[TOX_CLIENT_ID_SIZE] = 0;
 
-  uint8_t hex[TOX_CLIENT_ID_SIZE * 2 + 1];
-  memset(hex, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
+  uint8_t hex[TOX_CLIENT_ID_SIZE << 1 + 1];
+  memset(hex, 0, TOX_CLIENT_ID_SIZE << 1 + 1);
 
   int friend_num = 0;
 
@@ -1237,6 +1239,53 @@ ToxCore_file_data_remaining(ToxCore* self, PyObject* args)
   return PyLong_FromUnsignedLongLong(ret);
 }
 
+static PyObject*
+ToxCore_get_nospam(ToxCore* self, PyObject* args)
+{
+  CHECK_TOX(self);
+
+  uint32_t nospam = tox_get_nospam(self->tox);
+
+  return PyLong_FromUnsignedLongLong(nospam);
+}
+
+static PyObject*
+ToxCore_set_nospam(ToxCore* self, PyObject* args)
+{
+  CHECK_TOX(self);
+
+  uint32_t nospam = 0;
+
+  if (!PyArg_ParseTuple(args, "I", &nospam)) {
+    return NULL;
+  }
+
+  tox_set_nospam(self->tox, nospam);
+
+  Py_RETURN_NONE;
+}
+
+static PyObject*
+ToxCore_get_keys(ToxCore* self, PyObject* args)
+{
+  uint8_t public_key[TOX_CLIENT_ID_SIZE];
+  uint8_t secret_key[TOX_CLIENT_ID_SIZE];
+
+  uint8_t public_key_hex[TOX_CLIENT_ID_SIZE << 1 + 1];
+  uint8_t secret_key_hex[TOX_CLIENT_ID_SIZE << 1 + 1];
+
+  tox_get_keys(self->tox, public_key, secret_key);
+  bytes_to_hex_string(public_key, TOX_CLIENT_ID_SIZE, public_key_hex);
+  bytes_to_hex_string(secret_key, TOX_CLIENT_ID_SIZE, secret_key_hex);
+
+  PyObject* res = PyTuple_New(2);
+  PyTuple_SetItem(res, 0, PYSTRING_FromStringAndSize(public_key_hex,
+        TOX_CLIENT_ID_SIZE << 1));
+  PyTuple_SetItem(res, 1, PYSTRING_FromStringAndSize(secret_key_hex,
+        TOX_CLIENT_ID_SIZE << 1));
+  return res;
+}
+
 static PyObject*
 ToxCore_bootstrap_from_address(ToxCore* self, PyObject* args)
 {
@@ -1290,6 +1339,16 @@ ToxCore_kill(ToxCore* self, PyObject* args)
   Py_RETURN_NONE;
 }
 
+static PyObject*
+ToxCore_do_interval(ToxCore* self, PyObject* args)
+{
+  CHECK_TOX(self);
+
+  uint32_t interval = tox_do_interval(self->tox);
+
+  return PyLong_FromUnsignedLongLong(interval);
+}
+
 static PyObject*
 ToxCore_do(ToxCore* self, PyObject* args)
 {
@@ -1318,42 +1377,9 @@ ToxCore_save(ToxCore* self, PyObject* args)
 {
   CHECK_TOX(self);
 
-  PyObject* keyobj = Py_None;
-  uint8_t* key = NULL;
-  Py_ssize_t key_len = 0;
-
-  if (!PyArg_ParseTuple(args, "|O", &keyobj)) {
-    return NULL;
-  }
-
-  if (PYSTRING_Check(keyobj)) {
-    PyStringUnicode_AsStringAnsSize(keyobj, (char**)&key, &key_len);
-  } else if (keyobj != Py_None) {
-    PyErr_SetString(ToxOpError, "invalid passphrase type");
-    return NULL;
-  }
-
-  int size = 0;
-  if (key) {
-    size = tox_size_encrypted(self->tox);
-  } else {
-    size = tox_size(self->tox);
-  }
+  uint32_t size = tox_size(self->tox);
   uint8_t* buf = (uint8_t*)malloc(size);
-
-  if (buf == NULL) {
-    return PyErr_NoMemory();
-  }
-
-  if (key) {
-    if (tox_save_encrypted(self->tox, buf, key, key_len) == -1) {
-      PyErr_SetString(ToxOpError, "tox_save_encrypted(): failed");
-      free(buf);
-      return NULL;
-    }
-  } else {
-    tox_save(self->tox, buf);
-  }
+  tox_save(self->tox, buf);
 
   PyObject* res = PYBYTES_FromStringAndSize((const char*)buf, size);
   free(buf);
@@ -1366,28 +1392,13 @@ ToxCore_save_to_file(ToxCore* self, PyObject* args)
 {
   CHECK_TOX(self);
 
-  PyObject* keyobj = Py_None;
   char* filename = NULL;
-  uint8_t* key = NULL;
-  Py_ssize_t key_len = 0;
 
-  if (!PyArg_ParseTuple(args, "s|O", &filename, &keyobj)) {
+  if (!PyArg_ParseTuple(args, "s", &filename)) {
     return NULL;
   }
 
-  if (PYSTRING_Check(keyobj)) {
-    PyStringUnicode_AsStringAnsSize(keyobj, (char**)&key, &key_len);
-  } else if (keyobj != Py_None) {
-    PyErr_SetString(ToxOpError, "invalid passphrase type");
-    return NULL;
-  }
-
-  int size = 0;
-  if (key) {
-    size = tox_size_encrypted(self->tox);
-  } else {
-    size = tox_size(self->tox);
-  }
+  int size = tox_size(self->tox);
 
   uint8_t* buf = (uint8_t*)malloc(size);
 
@@ -1395,15 +1406,7 @@ ToxCore_save_to_file(ToxCore* self, PyObject* args)
     return PyErr_NoMemory();
   }
 
-  if (key) {
-    if (tox_save_encrypted(self->tox, buf, key, key_len) == -1) {
-      PyErr_SetString(ToxOpError, "tox_save_encrypted(): failed");
-      free(buf);
-      return NULL;
-    }
-  } else {
-    tox_save(self->tox, buf);
-  }
+  tox_save(self->tox, buf);
 
   FILE* fp = fopen(filename, "w");
 
@@ -1427,34 +1430,17 @@ ToxCore_load(ToxCore* self, PyObject* args)
 
   uint8_t* data = NULL;
   int length = 0;
-  PyObject* keyobj = Py_None;
-  uint8_t* key = NULL;
-  Py_ssize_t key_len = 0;
 
-  if (!PyArg_ParseTuple(args, "s#|O", &data, &length, &keyobj)) {
+  if (!PyArg_ParseTuple(args, "s#", &data, &length)) {
     PyErr_SetString(ToxOpError, "no data specified");
     return NULL;
   }
 
-  if (PYSTRING_Check(keyobj)) {
-    PyStringUnicode_AsStringAnsSize(keyobj, (char**)&key, &key_len);
-  } else if (keyobj != Py_None) {
-    PyErr_SetString(ToxOpError, "invalid passphrase type");
+  if (tox_load(self->tox, data, length) == -1) {
+    PyErr_SetString(ToxOpError, "tox_load(): load failed");
     return NULL;
   }
 
-  if (key) {
-    if (tox_load_encrypted(self->tox, data, length, key, key_len) == -1) {
-      PyErr_SetString(ToxOpError, "tox_load_encrypted(): load failed");
-      return NULL;
-    }
-  } else {
-    if (tox_load(self->tox, data, length) == -1) {
-      PyErr_SetString(ToxOpError, "tox_load(): load failed");
-      return NULL;
-    }
-  }
-
   Py_RETURN_NONE;
 }
 
@@ -1464,18 +1450,8 @@ ToxCore_load_from_file(ToxCore* self, PyObject* args)
   CHECK_TOX(self);
 
   char* filename = NULL;
-  PyObject* keyobj = Py_None;
-  uint8_t* key = NULL;
-  Py_ssize_t key_len = 0;
 
-  if (!PyArg_ParseTuple(args, "s|O", &filename, &keyobj)) {
-    return NULL;
-  }
-
-  if (PYSTRING_Check(keyobj)) {
-    PyStringUnicode_AsStringAnsSize(keyobj, (char**)&key, &key_len);
-  } else if (keyobj != Py_None) {
-    PyErr_SetString(ToxOpError, "invalid passphrase type");
+  if (!PyArg_ParseTuple(args, "s", &filename)) {
     return NULL;
   }
 
@@ -1500,18 +1476,10 @@ ToxCore_load_from_file(ToxCore* self, PyObject* args)
     return NULL;
   }
 
-  if (key) {
-    if (tox_load_encrypted(self->tox, data, length, key, key_len) == -1) {
-      PyErr_SetString(ToxOpError, "tox_load_encrypted(): load failed");
-      free(data);
-      return NULL;
-    }
-  } else {
-    if (tox_load(self->tox, data, length) == -1) {
-      PyErr_SetString(ToxOpError, "tox_load(): load failed");
-      free(data);
-      return NULL;
-    }
+  if (tox_load(self->tox, data, length) == -1) {
+    PyErr_SetString(ToxOpError, "tox_load(): load failed");
+    free(data);
+    return NULL;
   }
 
   free(data);
@@ -1925,6 +1893,25 @@ PyMethodDef Tox_methods[] = {
     "Give the number of bytes left to be sent/received. *send_receive* is "
     "0 for sending and 1 for receiving."
   },
+  {
+    "get_nospam", (PyCFunction)ToxCore_get_nospam,
+    METH_NOARGS,
+    "get_nospam()\n"
+    "get nospam part from ID"
+  },
+  {
+    "set_nospam", (PyCFunction)ToxCore_set_nospam,
+    METH_VARARGS,
+    "set_nospam(nospam)\n"
+    "set nospam part of ID. *nospam* should be of type uint32"
+  },
+  {
+    "get_keys", (PyCFunction)ToxCore_get_keys,
+    METH_NOARGS,
+    "get_keys()\n"
+    "Get the public and secret key from the Tox object. Return a tuple "
+    "(public_key, secret_key)"
+  },
   {
     "bootstrap_from_address", (PyCFunction)ToxCore_bootstrap_from_address,
     METH_VARARGS,
@@ -1942,6 +1929,11 @@ PyMethodDef Tox_methods[] = {
     "kill()\n"
     "Run this before closing shop."
   },
+  {
+    "do_interval", (PyCFunction)ToxCore_do_interval, METH_NOARGS,
+    "do_interval()\n"
+    "returns time (in ms) before the next tox_do() needs to be run on success."
+  },
   {
     "do", (PyCFunction)ToxCore_do, METH_NOARGS,
     "do()\n"
@@ -1954,22 +1946,22 @@ PyMethodDef Tox_methods[] = {
   },
   {
     "save", (PyCFunction)ToxCore_save, METH_VARARGS,
-    "save([passphrase=None])\n"
+    "save()\n"
     "Return messenger blob in str."
   },
   {
     "save_to_file", (PyCFunction)ToxCore_save_to_file, METH_VARARGS,
-    "save_to_file(filename[, passphrase=None])\n"
+    "save_to_file(filename)\n"
     "Save the messenger to a file."
   },
   {
     "load", (PyCFunction)ToxCore_load, METH_VARARGS,
-    "load(blob[, passphrase=None])\n"
+    "load(blob)\n"
     "Load the messenger from *blob*."
   },
   {
     "load_from_file", (PyCFunction)ToxCore_load_from_file, METH_VARARGS,
-    "load_from_file(filename[, passphrase=None])\n"
+    "load_from_file(filename)\n"
     "Load the messenger from file."
   },
   {NULL}
